@@ -22,8 +22,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """
 
-
-__version__ = "0.3.32"
+__version__ = "0.3.33"
 __author__ = "nagev"
 __license__ = "GPLv3"
 
@@ -55,6 +54,61 @@ if os.path.exists(os.path.join(os.path.expanduser("~"), ".pafydebug")):
     logging.basicConfig(level=logging.DEBUG)
 
 
+
+class g(object):
+
+    """ Class for holding vars / lambdas needed throughout the module. """
+
+    infoUrl = 'https://www.youtube.com/get_video_info'
+    infoUrlqs = 'video_id=%s&asv=3&el=detailpage&hl=en_US'
+    ua = ("Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64;"
+          "Trident/5.0)")
+    opener = build_opener()
+    opener.addheaders = [('User-Agent', ua)]
+    jsfuncs = {}
+    jsfunctimes = {}
+    funclife = 60 * 12
+    itags = {
+        '5': ('320x240', 'flv', "normal"),
+        '17': ('176x144', '3gp', "normal"),
+        '18': ('640x360', 'mp4', "normal"),
+        '22': ('1280x720', 'mp4', "normal"),
+        '34': ('640x360', 'flv', "normal"),
+        '35': ('854x480', 'flv', "normal"),
+        '36': ('320x240', '3gp', "normal"),
+        '37': ('1920x1080', 'mp4', "normal"),
+        '38': ('4096x3072', 'superHD', "normal"),
+        '43': ('640x360', 'webm', "normal"),
+        '44': ('854x480', 'webm', "normal"),
+        '45': ('1280x720', 'webm', "normal"),
+        '46': ('1920x1080', 'webm', "normal"),
+        '82': ('640x360-3D', 'mp4', "normal"),
+        '84': ('1280x720-3D', 'mp4', "normal"),
+        '100': ('640x360-3D', 'webm', "normal"),
+        '102': ('1280x720-3D', 'webm', "normal"),
+        '133': ('426x240', 'm4v', 'video'),
+        '134': ('640x360', 'm4v', 'video'),
+        '135': ('854x480', 'm4v', 'video'),
+        '136': ('1280x720', 'm4v', 'video'),
+        '137': ('1920x1080', 'm4v', 'video'),
+        '138': ('4096x3072', 'm4v', 'video'),
+        '139': ('48k', 'm4a', 'audio'),
+        '140': ('128k', 'm4a', 'audio'),
+        '141': ('256k', 'm4a', 'audio'),
+        '160': ('256x144', 'm4v', 'video'),
+        '171': ('128k', 'ogg', 'audio'),
+        '172': ('192k', 'ogg', 'audio'),
+        '242': ('360x240', 'webm', 'normal'),
+        '243': ('480x360', 'webm', 'normal'),
+        '244': ('640x480', 'webm', 'normal'),
+        '245': ('640x480', 'webm', 'normal'),
+        '246': ('640x480', 'webm', 'normal'),
+        '247': ('720x480', 'webm', 'normal'),
+        '248': ('unknown', 'unknown', 'unknown'),
+        '264': ('1920x1080', 'm4v', 'video')
+    }
+
+
 def _extract_function_from_js(name, js):
     """ Find a function definition called `name` and extract components.
 
@@ -62,11 +116,19 @@ def _extract_function_from_js(name, js):
 
     """
 
-    fpattern = r'function\s%s\(((?:\w+,?)+)\)\{([^}]+)\}'
+
+    # use cached js function
+    if g.jsfuncs.get(name) and g.jsfunctimes[name] > time.time() - g.funclife:
+        return g.jsfuncs.get(name)
+
+    fpattern = r'function\s+%s\(((?:\w+,?)+)\)\{([^}]+)\}'
     m = re.search(fpattern % re.escape(name), js)
     args, body = m.groups()
-    logging.debug("extracted function %s(%s){%s};", name, args, body)
-    return {'name': name, 'parameters': args.split(","), 'body': body}
+    #logging.debug("extracted function %s(%s){%s};", name, args, body)
+    func = {'name': name, 'parameters': args.split(","), 'body': body}
+    g.jsfuncs[name] = func
+    g.jsfunctimes[name] = time.time()
+    return func
 
 
 def _getval(val, argsdict):
@@ -196,79 +258,40 @@ def new(url, callback=None):
     return Pafy(url, callback=callback)
 
 
+new.safeint = lambda x: int(x) if x.isdigit() else x
+
+
 class Stream(object):
 
     """ YouTube video stream class. """
 
-    itags = {
-        '5': ('320x240', 'flv', "normal"),
-        '17': ('176x144', '3gp', "normal"),
-        '18': ('640x360', 'mp4', "normal"),
-        '22': ('1280x720', 'mp4', "normal"),
-        '34': ('640x360', 'flv', "normal"),
-        '35': ('854x480', 'flv', "normal"),
-        '36': ('320x240', '3gp', "normal"),
-        '37': ('1920x1080', 'mp4', "normal"),
-        '38': ('4096x3072', 'superHD', "normal"),
-        '43': ('640x360', 'webm', "normal"),
-        '44': ('854x480', 'webm', "normal"),
-        '45': ('1280x720', 'webm', "normal"),
-        '46': ('1920x1080', 'webm', "normal"),
-        '82': ('640x360-3D', 'mp4', "normal"),
-        '84': ('1280x720-3D', 'mp4', "normal"),
-        '100': ('640x360-3D', 'webm', "normal"),
-        '102': ('1280x720-3D', 'webm', "normal"),
-        '133': ('426x240', 'm4v', 'video'),
-        '134': ('640x360', 'm4v', 'video'),
-        '135': ('854x480', 'm4v', 'video'),
-        '136': ('1280x720', 'm4v', 'video'),
-        '137': ('1920x1080', 'm4v', 'video'),
-        '138': ('4096x3072', 'm4v', 'video'),
-        '139': ('48k', 'm4a', 'audio'),
-        '140': ('128k', 'm4a', 'audio'),
-        '141': ('256k', 'm4a', 'audio'),
-        '160': ('256x144', 'm4v', 'video'),
-        '171': ('128k', 'ogg', 'audio'),
-        '172': ('192k', 'ogg', 'audio'),
-        '242': ('360x240', 'webm', 'normal'),
-        '243': ('480x360', 'webm', 'normal'),
-        '244': ('640x480', 'webm', 'normal'),
-        '245': ('640x480', 'webm', 'normal'),
-        '246': ('640x480', 'webm', 'normal'),
-        '247': ('720x480', 'webm', 'normal'),
-        '248': ('unknown', 'unknown', 'unknown'),
-        '264': ('1920x1080', 'm4v', 'video')
-    }
-
-    def __init__(self, sm, opener, title="ytvid", js=None):
+    def __init__(self, sm, title="ytvid", js=None):
 
         self.url = sm['url'][0]
 
         if sm.get("s"):
             sm['sig'] = [_decodesig(sm['s'][0], js)]
-            logging.debug("Calculated decrypted sig: " + sm['sig'][0])
-
-        if not "signature=" in self.url:
-            self.url += '&signature=' + sm['sig'][0]
+            #logging.debug("Calculated decrypted sig: " + sm['sig'][0])
 
         if not "ratebypass=" in self.url:
             self.url = self.url + "&ratebypass=yes"
 
+        if not "signature=" in self.url:
+            self.url += '&signature=' + sm['sig'][0]
+
         self.itag = sm['itag'][0]
         self.threed = 'stereo3d' in sm and sm['stereo3d'][0] == '1'
-        self.resolution = self.itags[self.itag][0]
+        self.resolution = g.itags[self.itag][0]
         self.dimensions = tuple(self.resolution.split("-")[0].split("x"))
-        safeint = lambda x: int(x) if x.isdigit() else x
-        self.dimensions = tuple(map(safeint, self.dimensions))
+        self.dimensions = tuple(map(new.safeint, self.dimensions))
         self.vidformat = sm['type'][0].split(';')[0]
         self.quality = self.resolution
-        self.extension = self.itags[self.itag][1]
+        self.extension = g.itags[self.itag][1]
         self.title = title
         self.filename = self.title + "." + self.extension
         self.fsize = None
-        self._opener = opener
         self.bitrate = self.rawbitrate = None
-        self.mediatype = self.itags[self.itag][2]
+        self.mediatype = g.itags[self.itag][2]
 
         if self.mediatype == "audio":
             self.bitrate = self.resolution
@@ -287,9 +310,8 @@ class Stream(object):
         if not self.fsize:
 
             try:
-                opener = self._opener
                 cl = "content-length"
-                self.fsize = int(opener.open(self.url).headers[cl])
+                self.fsize = int(g.opener.open(self.url).headers[cl])
 
             except (HTTPError, URLError):
                 self.fsize = 0
@@ -302,8 +324,8 @@ class Stream(object):
         """ Download.  Use quiet=True to supress output. Return filename. """
 
         status_string = ('  {:,} Bytes [{:.2%}] received. Rate: [{:4.0f} '
-                         'kbps].  ETA: [{:.0f} secs]')
-        response = self._opener.open(self.url)
+                         'KB/s].  ETA: [{:.0f} secs]')
+        response = g.opener.open(self.url)
         total = int(response.info()['Content-Length'].strip())
         chunksize, bytesdone, t0 = 16384, 0, time.time()
         fname = filepath or self.filename
@@ -369,18 +391,18 @@ class Pafy(object):
 
         return "\n".join(["%s: %s" % (k, info.get(k, "")) for k in keys])
 
-    def get_js(self, opener):
+    def get_js(self):
         """ Get location of html5player javascript file and fetch.
 
         Return javascript as string and args.
 
         """
 
-        logging.debug("call to get js")
-
         if not self.js or not self.xargs:
             watchurl = "https://www.youtube.com/watch?v=" + self.videoid
-            watchinfo = opener.open(watchurl).read().decode("UTF-8")
+            new.callback("Fetching watch?v page")
+            watchinfo = g.opener.open(watchurl).read().decode("UTF-8")
+            new.callback("watch?v page fetched")
             m = re.search(r';ytplayer.config = ({.*?});', watchinfo)
 
             try:
@@ -395,23 +417,32 @@ class Pafy(object):
             if html5player.startswith("//"):
                 html5player = "https:" + html5player
 
-            logging.debug("opening js url")
-            js = opener.open(html5player).read().decode("UTF-8")
+            # check whether we already have this html5js file
+            h5 = g.jsfuncs.get(html5player)
+
+            if h5 and g.jsfunctimes[html5player] > time.time() - g.funclife:
+                js = h5
+
+            else:
+                new.callback("Fetching html5player page")
+                js = g.opener.open(html5player).read().decode("UTF-8")
+                new.callback("html5player page fetched")
+                g.jsfunctimes[html5player] = time.time()
+                g.jsfuncs[html5player] = js
+
             self.js, self.xargs = js, args
 
         return(self.js, self.xargs)
 
-    def getstreammap(self, allinfo, key, opener):
+    def getstreammap(self, allinfo, key):
         """ get stream map. Return stream map and javascript."""
 
         js = self.js
         streamMap = allinfo[key][0].split(',')
         smap = [parse_qs(sm) for sm in streamMap]
         if smap[0].get("s"):
-            logging.debug("encrypted sig")
-            new.callback("Getting javascript data..")
-            js, args = self.get_js(opener)
-            new.callback("Got javascript")
+            new.callback("Encrypted signature detected")
+            js, args = self.get_js()
             streamMap = args[key].split(",")
             smap = [parse_qs(sm) for sm in streamMap]
         return(smap, js)
@@ -426,7 +457,7 @@ class Pafy(object):
         if not self.gdata:
             url = "https://gdata.youtube.com/feeds/api/videos/%s?v=2"
             url = url % self.videoid
-            self.gdata = self._opener.open(url).read()
+            self.gdata = g.opener.open(url).read()
         return self.gdata
 
     @property
@@ -460,8 +491,7 @@ class Pafy(object):
 
     def __init__(self, video_url, callback=None):
 
-        infoUrl = 'https://www.youtube.com/get_video_info?video_id='
-        ok = ("a-zA-Z0-9_-",) * 3
+        ok = (r"\w-",) * 3
         regx = re.compile(r'(?:^|[^%s]+)([%s]{11})(?:[^%s]+|$)' % ok)
         m = regx.search(video_url)
 
@@ -470,14 +500,8 @@ class Pafy(object):
             raise RuntimeError(err % video_url)
 
         vidid = m.group(1)
-        infoUrl += vidid + "&asv=3&el=detailpage&hl=en_US"
-        opener = build_opener()
-        ua = ("Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64;"
-              "Trident/5.0)")
-
-        opener.addheaders = [('User-Agent', ua)]
-        self.keywords = ""
-        allinfo = parse_qs(decode_if_py3(opener.open(infoUrl).read()))
+        info_url = "?".join([g.infoUrl, g.infoUrlqs % vidid])
+        allinfo = parse_qs(decode_if_py3(g.opener.open(info_url).read()))
 
         if allinfo['status'][0] == "fail":
             reason = allinfo['reason'][0] or "Bad video argument"
@@ -490,6 +514,7 @@ class Pafy(object):
             new.callback = lambda x: None
 
         f = lambda x: allinfo.get(x, ["unknown"])[0]
+        self.keywords = ""
         self.gdata = None
         self.xargs = None
         self.js = None
@@ -506,6 +531,7 @@ class Pafy(object):
         self._description = None
         self._category = None
         self.keywords = self.bigthumb = self.bigthumbhd = None
+        self.expires = int(time.time()) + (60 * 60 * 4)
 
         if 'keywords' in allinfo:
             self.keywords = f('keywords').split(',')
@@ -516,22 +542,18 @@ class Pafy(object):
         if allinfo.get('iurlmaxres'):
             self.bigthumbhd = f('iurlmaxres')
 
-        self._opener = opener
-
         try:
-            smap, js = self.getstreammap(
-                allinfo, 'url_encoded_fmt_stream_map', opener)
+            smap, js = self.getstreammap(allinfo, 'url_encoded_fmt_stream_map')
 
         except KeyError:
             raise IOError("Can't get video stream info")
 
-        self.streams = [Stream(sm, opener, self.title, js) for sm in smap]
+        self.streams = [Stream(sm, self.title, js) for sm in smap]
         self.videostreams = self.audiostreams = []
 
         if "adaptive_fmts" in allinfo:
-            smap_adpt, js = self.getstreammap(allinfo, 'adaptive_fmts', opener)
-            self.streams_ad = [Stream(sm, opener, self.title, js) for sm in
-                               smap_adpt]
+            smap_adpt, js = self.getstreammap(allinfo, 'adaptive_fmts')
+            self.streams_ad = [Stream(sm, self.title, js) for sm in smap_adpt]
             self.audiostreams = [x for x in self.streams_ad if x.bitrate]
             self.videostreams = [x for x in self.streams_ad if not x.bitrate]
             m4astreams = [x for x in self.audiostreams if x.extension == "m4a"]
@@ -539,7 +561,6 @@ class Pafy(object):
             self.m4astreams, self.oggstreams = m4astreams, oggstreams
 
         self.allstreams = self.streams + self.videostreams + self.audiostreams
-        self.expires = int(time.time()) + (60 * 60 * 4.8)
 
     def getbest(self, preftype="any", ftypestrict=True):
         """
